package com.flyfish.framework.bean;

import com.flyfish.framework.constant.Frameworks;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import reactor.core.publisher.Mono;

import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.function.Function;

/**
 * 请求结果
 *
 * @author Mr.Wang
 */
@SuppressWarnings("unchecked")
public class Result<T> {

    public static final String SUCCESS = "0";

    public static final String FAIL = "-1";

    public static final String MSG_SUCCESS = "请求成功";

    public static final String MSG_FAIL = "系统异常！";

    public static final String MSG_NO_DATA = "未查询到任何数据！";

    private T data;

    private String code;

    private String msg;

    public Result(String code, String msg, T data) {
        this.code = code;
        this.msg = msg;
        this.data = data;
    }

    public Result(String code, String msg) {
        this.code = code;
        this.msg = msg;
        this.data = (T) Collections.emptyMap();
    }

    public static <T> Result<T> accept(T data) {
        if (null == data) {
            return new Result<>(FAIL, MSG_NO_DATA);
        }
        return new Result<>(SUCCESS, MSG_SUCCESS, data);
    }

    public static <T> Mono<Result<T>> just(T data) {
        return Mono.just(accept(data));
    }

    public static <T> Mono<Result<List<T>>> just(Page<T> page) {
        return Mono.just(accept(page));
    }

    public static <T> Mono<Result<T>> just() {
        return Mono.just(ok());
    }

    /**
     * 单个元素，附带分页信息，分页数据写死
     *
     * @param data 数据
     * @param <T>  泛型
     * @return 结果
     */
    public static <T> Result<List<T>> single(T data) {
        Page<T> page = new PageImpl<>(Collections.singletonList(data), PageResult.DEFAULT_PAGE, 1);
        return accept(page);
    }

    public static <T> Result<List<T>> accept(List<T> data, Pageable pageable, int total) {
        Page<T> page = new PageImpl<>(data, pageable, total);
        return accept(page);
    }

    public static <T> Result<List<T>> accept(Page<T> page) {
        if (null == page) {
            return new PageResult<>(FAIL, MSG_NO_DATA);
        }
        return new PageResult<>(SUCCESS, MSG_SUCCESS, page);
    }

    public static <T> Result<T> error() {
        return new Result<>(FAIL, MSG_FAIL, null);
    }

    public static <T> Result<T> ok() {
        return empty();
    }

    public static <T> Result<T> ok(T data) {
        return new Result<>(SUCCESS, MSG_SUCCESS, data);
    }

    public static <T> Result<T> empty() {
        return new Result<>(SUCCESS, MSG_SUCCESS, null);
    }

    public static <T> Result<List<T>> emptyList() {
        return new Result<>(SUCCESS, MSG_SUCCESS, Collections.emptyList());
    }

    public static <T> Result<List<T>> emptyPage() {
        return new PageResult<>(SUCCESS, MSG_SUCCESS, Page.empty());
    }

    public static <T> Result<T> notFound() {
        return new Result<>(FAIL, MSG_NO_DATA, null);
    }

    public static <T> Result<T> error(String errMsg) {
        return new Result<>(FAIL, errMsg, null);
    }

    public Result<T> end(Runnable runnable) {
        runnable.run();
        return this;
    }

    public Result<Void> toVoid() {
        return (Result<Void>) this;
    }

    public Optional<T> optional() {
        return isSuccess() ? Optional.ofNullable(data) : Optional.empty();
    }

    public <R> Result<R> map(Function<T, R> function) {
        if (this.data != null) {
            this.data = (T) function.apply(this.data);
            return (Result<R>) this;
        }
        return (Result<R>) this;
    }

    public Result<T> orElse(Result<T> other) {
        if (null == this.data) {
            return other;
        }
        return this;
    }

    public Result<T> filter(Function<T, Boolean> function) {
        if (!function.apply(this.data)) {
            this.data = null;
        }
        return this;
    }

    public boolean isSuccess() {
        return SUCCESS.equals(this.code);
    }

    public T data() {
        return this.data;
    }

    public T getData() {
        return Frameworks.config.isResultStyle() ? null : data;
    }

    public void setData(T data) {
        this.data = data;
    }

    public T getResult() {
        return Frameworks.config.isResultStyle() ? data : null;
    }

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public String getMsg() {
        return Frameworks.config.isResultStyle() ? null : msg;
    }

    public void setMsg(String msg) {
        this.msg = msg;
    }

    public String getMessage() {
        return Frameworks.config.isResultStyle() ? msg : null;
    }
}
