package com.flyfish.framework.logging.service;

import com.flyfish.framework.annotations.Operation;
import com.flyfish.framework.beans.meta.RestBean;
import com.flyfish.framework.context.UserContext;
import com.flyfish.framework.domain.base.Domain;
import com.flyfish.framework.logging.domain.DomainDescriptions;
import com.flyfish.framework.utils.ReflectionUtils;
import lombok.AccessLevel;
import lombok.Data;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ClassUtils;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.data.util.CastUtils;

import java.util.Optional;

/**
 * 日志上下文，线程协作好帮手
 *
 * @author wangyu
 */
@Data
@Slf4j
public class LogContext {

    private static final LogContext UN_SUPPORT = new LogContext();

    private long startTime;

    private long endTime;

    @Setter(AccessLevel.NONE)
    private Throwable error;

    @Setter(AccessLevel.NONE)
    private Object result;

    private Object[] args;

    private String module;

    private String business;

    private String signature;

    private String user;

    private boolean valid = false;

    public static LogContext unSupport() {
        return UN_SUPPORT;
    }

    /**
     * 通过切入点创建
     *
     * @param joinPoint 切入点
     * @return 结果
     */
    public static LogContext of(ProceedingJoinPoint joinPoint) {
        Signature signature = joinPoint.getSignature();
        if (signature instanceof MethodSignature) {
            MethodSignature methodSignature = (MethodSignature) signature;
            Operation operation = AnnotationUtils.findAnnotation(methodSignature.getMethod(), Operation.class);
            if (null != operation) {
                Class<?> targetClass = joinPoint.getTarget().getClass();
                LogContext context = new LogContext();
                context.startTime = System.currentTimeMillis();
                context.module = determineModule(operation, targetClass);
                context.business = operation.value();
                context.signature = ClassUtils.getCanonicalName(targetClass) + "." + methodSignature.getName();
                context.args = joinPoint.getArgs();
                context.valid = true;
                log.info("监测到支持的切点：{}, {}", context.module, operation.value());
                return context;
            }
        }
        return UN_SUPPORT;
    }

    /**
     * 确定模块
     *
     * @param operation 模块
     * @return 结果
     */
    private static String determineModule(Operation operation, Class<?> targetClass) {
        String module = operation.module();
        if (StringUtils.isBlank(module)) {
            return ReflectionUtils.getGenericType(targetClass)
                    .flatMap(clazz -> {
                        DomainDescriptions description = DomainDescriptions.getByClass(CastUtils.cast(clazz));
                        if (DomainDescriptions.DEFAULT != description) {
                            return Optional.of(description.getName());
                        }
                        return Optional.ofNullable(AnnotationUtils.findAnnotation(clazz, RestBean.class))
                                .map(RestBean::name);
                    })
                    .orElse("未知模块");
        }
        return module;
    }

    public void setResult(Object result) {
        if (valid) {
            this.result = result;
        }
    }

    public void setError(Throwable error) {
        if (valid) {
            this.error = error;
        }
    }

    public boolean isSuccess() {
        return null == error;
    }

    public void end() {
        this.user = UserContext.sharedContext().map(UserContext::currentUser).map(Domain::getName).orElse("未知");
        this.endTime = System.currentTimeMillis();
    }
}
