package com.flyfish.framework.controller;

import com.flyfish.framework.annotations.Operation;
import com.flyfish.framework.bean.Result;
import com.flyfish.framework.bean.SyncVo;
import com.flyfish.framework.configuration.annotations.CurrentUser;
import com.flyfish.framework.configuration.annotations.PagedQuery;
import com.flyfish.framework.configuration.annotations.ValidRequestBody;
import com.flyfish.framework.constant.ReactiveConstants;
import com.flyfish.framework.context.UserContext;
import com.flyfish.framework.domain.base.Domain;
import com.flyfish.framework.domain.base.Qo;
import com.flyfish.framework.domain.po.User;
import com.flyfish.framework.service.BaseReactiveService;
import com.flyfish.framework.service.BaseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Mono;

import java.util.Arrays;
import java.util.List;

/**
 * 通用RESTful的控制器，用于动态提供基础的RESTful接口
 *
 * @author wangyu
 * @create 2017-06-15 8:48
 */
public abstract class BaseController<T extends Domain, Q extends Qo<T>> implements SafeController {

    @Autowired
    protected BaseService<T> service;
    @Autowired(required = false)
    protected BaseReactiveService<T> reactiveService;
    @Autowired
    protected UserContext userContext;

    @SuppressWarnings("unchecked")
    public <S extends BaseService<T>> S getService() {
        return (S) service;
    }

    @GetMapping("/exists")
    public Result<Boolean> exists(@PagedQuery Q qo) {
        return Result.accept(service.count(qo) != 0);
    }

    @GetMapping("/count")
    public Result<Long> count(@PagedQuery Q qo) {
        return Result.accept(service.count(qo));
    }

    @PostMapping("")
    @Operation.Create
    public Result<T> create(@ValidRequestBody T entity, @CurrentUser User user) {
        userContext.setUser(user);
        return Result.accept(service.create(entity));
    }

    @GetMapping("{id}")
    public Result<T> get(@PathVariable String id) {
        return service.getById(id).map(Result::accept).orElse(Result.notFound());
    }

    @PutMapping("{id}")
    @Operation.Update
    public Result<T> update(@ValidRequestBody T entity, @CurrentUser User user) {
        userContext.setUser(user);
        return Result.accept(service.updateSelectiveById(entity));
    }

    @PatchMapping("{id}")
    @Operation.Update
    public Result<T> patch(@RequestBody T entity, @CurrentUser User user) {
        userContext.setUser(user);
        return Result.accept(service.updateSelectiveById(entity));
    }

    @PutMapping("")
    @Operation.UpdateAll
    public Result<List<T>> updateList(@RequestBody List<T> list, @CurrentUser User user) {
        userContext.setUser(user);
        return Result.accept(service.updateBatch(list));
    }

    @PutMapping("/sync")
    @Operation.Sync
    public Result<SyncVo<T>> syncList(@RequestBody List<T> list, @CurrentUser User user) {
        userContext.setUser(user);
        return Result.accept(service.sync(list));
    }

    @DeleteMapping("{id}")
    @Operation.Delete
    public Result<T> remove(@PathVariable List<String> id, @CurrentUser User user) {
        userContext.setUser(user);
        service.deleteBatchByIds(id);
        return Result.ok();
    }

    @GetMapping("/all")
    public Result<List<T>> all(@PagedQuery Q qo) {
        if (qo.isEmpty()) {
            return Result.accept(service.getAll());
        }
        return Result.accept(service.getList(qo));
    }

    @GetMapping("")
    public Result<List<T>> list(@PagedQuery Q qo) {
        if (null != qo.getPageable()) {
            return Result.accept(service.getPageList(qo));
        }
        return Result.accept(service.getList(qo));
    }

    /**
     * reactive接口
     */
    @PostMapping(value = "", headers = ReactiveConstants.USE_REACTIVE)
    public Mono<Result<T>> reactiveCreate(@ValidRequestBody T entity) {
        return reactiveService.create(entity).map(Result::accept);
    }

    @GetMapping(value = "{id}", headers = ReactiveConstants.USE_REACTIVE)
    public Mono<Result<T>> reactiveGet(@PathVariable String id) {
        return reactiveService.getById(id)
                .map(Result::accept)
                .defaultIfEmpty(Result.notFound());
    }

    @PutMapping(value = "{id}", headers = ReactiveConstants.USE_REACTIVE)
    public Mono<Result<T>> reactiveUpdate(@ValidRequestBody T entity) {
        return reactiveService.updateById(entity)
                .map(Result::accept)
                .defaultIfEmpty(Result.notFound());
    }

    @DeleteMapping(value = "{id}", headers = ReactiveConstants.USE_REACTIVE)
    public Mono<Result<T>> reactiveRemove(@PathVariable String id) {
        return reactiveService.deleteById(id).map(item -> Result.ok());
    }

    @GetMapping(value = "/all", headers = ReactiveConstants.USE_REACTIVE)
    public Mono<Result<List<T>>> reactiveAll() {
        return reactiveService.getAll()
                .collectList()
                .map(Result::accept)
                .defaultIfEmpty(Result.emptyList());
    }

    @GetMapping(value = "", headers = ReactiveConstants.USE_REACTIVE)
    public Mono<Result<List<T>>> reactiveList(@PagedQuery Q qo) {
        if (null != qo.getPageable()) {
            return reactiveService.getPageList(qo)
                    .map(Result::accept)
                    .defaultIfEmpty(Result.emptyPage());
        }
        return reactiveService.getList(qo).collectList()
                .map(Result::accept)
                .defaultIfEmpty(Result.emptyList());
    }

}
